﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Sample.Models;
using Syncfusion.OfficeChart;
using Syncfusion.DocIO;
using Syncfusion.DocIORenderer;
using Syncfusion.Pdf;
using Syncfusion.DocIO.DLS;
using System.Threading;
using ExportImageFormat = Syncfusion.OfficeChart.ExportImageFormat;
using Syncfusion.Pdf.Barcode;
using System.Drawing;
using Syncfusion.Pdf.Graphics;
using System.Drawing.Imaging;

namespace Sample.Controllers
{
    public class HomeController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        private readonly IHostingEnvironment _hostingEnvironment;

        public HomeController(IHostingEnvironment hostingEnvironment)
        {
            _hostingEnvironment = hostingEnvironment;
        }

        [HttpPost]

        public void ConvertToPDF()
        {

                //Get the file path of the Word document
                string inputFilePath = _hostingEnvironment.WebRootPath + @"/barcodedoc.docx";
                //Open the file as Stream
                FileStream docStream = new FileStream(inputFilePath, FileMode.Open, FileAccess.Read);
                //Loads file stream into Word document
                WordDocument wordDocument = new WordDocument(docStream, Syncfusion.DocIO.FormatType.Automatic);

            #region Perform based on IF Field value
            //Iterates into document elements and gets IF fields in document.
           ConvertFieldToBarcodeImage(wordDocument);

            #endregion
                //Instantiation of DocIORenderer for Word to PDF conversion
                DocIORenderer render = new DocIORenderer();
                //Converts Word document into PDF document
                PdfDocument pdfDocument = render.ConvertToPDF(wordDocument);
                //Releases all resources used by the Word document and DocIO Renderer objects
                render.Dispose();
                wordDocument.Dispose();
                FileStream fileStream = new FileStream(_hostingEnvironment.WebRootPath + @"/Result.pdf", FileMode.Create);
                pdfDocument.Save(fileStream);
                fileStream.Position = 0;
                //Closes the instance of PDF document object
                pdfDocument.Close(true);

        }

        #region Iterates into document.
        /// <summary>
        /// Iterates to document elements and get fields.
        /// </summary>
        /// <param name="document">Input Word document.</param>
        /// <param name="fieldType">Type of the field to find in document.</param>
        private void ConvertFieldToBarcodeImage(WordDocument document)
        {
            //Iterates each section and get the tables.
            foreach (WSection section in document.Sections)
            {
                IterateTextBody(section.Body, FieldType.FieldIf);
            }

        }

        /// <summary>
        /// Iterates into body items.
        /// </summary>
        private void IterateTextBody(WTextBody textBody, FieldType fieldType)
        {
            for (int i = 0; i < textBody.ChildEntities.Count; i++)
            {
                //IEntity is the basic unit in DocIO DOM.                 
                IEntity bodyItemEntity = textBody.ChildEntities[i];

                //A Text body has 3 types of elements - Paragraph, Table, and Block Content Control
                //Decides the element type by using EntityType
                switch (bodyItemEntity.EntityType)
                {
                    case EntityType.Paragraph:
                        WParagraph paragraph = bodyItemEntity as WParagraph;
                        //Iterates into paragraph items.
                        IterateParagraph(paragraph.Items, fieldType);
                        break;

                    case EntityType.Table:
                        //Table is a collection of rows and cells
                        //Iterates through table's DOM
                        IterateTable(bodyItemEntity as WTable, fieldType);
                        break;

                    case EntityType.BlockContentControl:
                        BlockContentControl blockContentControl = bodyItemEntity as BlockContentControl;
                        //Iterates to the body items of Block Content Control.
                        IterateTextBody(blockContentControl.TextBody, fieldType);
                        break;
                }
            }
        }

        /// <summary>
        /// Iterates into paragraph items.
        /// </summary>
        /// <param name="paragraph">The paragraph.</param>
        /// <param name="fieldType">Type of field.</param>
        private void IterateParagraph(ParagraphItemCollection paraItems, FieldType fieldType)
        {
            for (int i = 0; i < paraItems.Count; i++)
            {
                if (paraItems[i] is WField)
                {
                    WField field = paraItems[i] as WField;
                    if(field.FieldCode.TrimStart().StartsWith("DISPLAYBARCODE"))
                    {
                        string barCodeValue = field.FieldValue.Split(' ')[0];
                        MemoryStream imageStream = GetAsImage(barCodeValue);
                        WPicture picture = new WPicture(paraItems.Document);
                        picture.LoadImage(imageStream);
                        paraItems.RemoveAt(i);
                        paraItems.Insert(i, picture);
                    }
                }
                else if (paraItems[i] is WTextBox)
                {
                    //If paragraph item is textbox, iterates into textbody of textbox.
                    WTextBox textBox = paraItems[i] as WTextBox;
                    IterateTextBody(textBox.TextBoxBody, fieldType);
                }
                else if (paraItems[i] is Shape)
                {
                    //If paragraph item is shape, iterates into textbody of shape.
                    Shape shape = paraItems[i] as Shape;
                    IterateTextBody(shape.TextBody, fieldType);
                }
                else if (paraItems[i] is InlineContentControl)
                {
                    //If paragraph item is inline content control, iterates into its item.
                    InlineContentControl inlineContentControl = paraItems[i] as InlineContentControl;
                    IterateParagraph(inlineContentControl.ParagraphItems, fieldType);
                }
            }
        }
        /// <summary>
        /// Iterates into table.
        /// </summary>
        /// <param name="table">The table.</param>
        /// <param name="fieldType">Type of Field.</param>
        private void IterateTable(WTable table, FieldType fieldType)
        {
            //Iterates the row collection in a table
            foreach (WTableRow row in table.Rows)
            {
                //Iterates the cell collection in a table row
                foreach (WTableCell cell in row.Cells)
                {
                    IterateTextBody(cell, fieldType);
                }
            }
        }
        /// <summary>
        /// Gets the text from the field to create Barcode image
        /// </summary>
        /// <param name="text">Text from the field</param>
        /// <returns></returns>
        private MemoryStream GetAsImage(string text)
        {
            PdfCode39Barcode barcode = new PdfCode39Barcode();

            //Set the height and text for barcode
            barcode.Bounds = new Syncfusion.Drawing.RectangleF(72,72,720,250);
            barcode.TextDisplayLocation = TextLocation.None;
            barcode.Text = text.Trim('*');
            
            //Convert the barcode to image
            Image barcodeImage = barcode.ToImage(new SizeF());
            //Create a new instance of memory stream
            MemoryStream stream = new MemoryStream();
            
            //Save image to stream.
            barcodeImage.Save(stream, ImageFormat.Png);
            stream.Position = 0;
            return stream;
        }
        #endregion

        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
